﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using UnivateProperties_API.Context;
using UnivateProperties_API.Model.Region;

namespace UnivateProperties_API.Repository.Region
{
    public class ProvinceRepository : IRepository<Province>
    {
        private readonly DataContext dBContext;

        public ProvinceRepository(DataContext _dBContext)
        {
            dBContext = _dBContext;
        }

        public List<Province> Get(Func<Province, bool> where)
        {
            return dBContext.Provinces.Where(where).ToList();
        }

        public List<Province> GetAll()
        {
            return dBContext.Provinces.OrderBy(p => p.Description).ToList();
        }

        public Province GetDetailed(Func<Province, bool> first)
        {
            return dBContext.Provinces.FirstOrDefault(first);
        }

        public List<Province> GetDetailedAll()
        {
            //TODO: remove NotImplemented
            throw new NotImplementedException();
        }

        public void Insert(Province item)
        {
            dBContext.Provinces.Add(item);
            Save();
        }

        public void Insert(IEnumerable<Province> items)
        {
            foreach (var item in items)
            {
                dBContext.Provinces.Add(item);
                Save();
            }
        }

        public void Remove(Province item)
        {
            dBContext.Provinces.Remove(item);
            Save();
        }

        public void Remove(IEnumerable<Province> items)
        {
            foreach (var item in items)
            {
                dBContext.Provinces.Remove(item);
            }
            Save();
        }

        public void RemoveAtId(int item)
        {
            var province = Get(x => x.Id == item).FirstOrDefault();
            if (province != null)
            {
                dBContext.Provinces.Remove(province);
                Save();
            }
        }

        public void Save()
        {
            dBContext.SaveChanges();
        }

        public void Update(Province item)
        {
            dBContext.Entry(item).State = EntityState.Modified;
            Save();
        }
    }
}