﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using UnivateProperties_API.Context;
using UnivateProperties_API.Model.Timeshare;

namespace UnivateProperties_API.Repository.Timeshare
{
    public class SeasonRepository : IRepository<Season>
    {
        private readonly DataContext _dbContext;

        public SeasonRepository(DataContext dbContext)
        {
            _dbContext = dbContext;
        }

        public List<Season> Get(Func<Season, bool> where)
        {
            return _dbContext.Seasons.Where(where).ToList();
        }

        public List<Season> GetAll()
        {
            return _dbContext.Seasons.ToList();
        }

        public Season GetDetailed(Func<Season, bool> first)
        {
            var item = _dbContext.Seasons.FirstOrDefault(first);
            return item;
        }

        public List<Season> GetDetailedAll()
        {
            return GetAll();
        }

        public void Insert(Season item)
        {
            _dbContext.Add(item);
            Save();
        }

        public void Insert(IEnumerable<Season> items)
        {
            foreach (var item in items)
            {
                _dbContext.Add(item);
            }
            Save();
        }

        public void Remove(Season item)
        {
            var i = _dbContext.Seasons.Find(item);
            _dbContext.Seasons.Remove(i);
            Save();
        }

        public void Remove(IEnumerable<Season> items)
        {
            foreach (var item in items)
            {
                var i = _dbContext.Seasons.Find(item);
                _dbContext.Seasons.Remove(i);
            }
            Save();
        }

        public void RemoveAtId(int item)
        {
            var i = _dbContext.Seasons.Find(item);
            _dbContext.Seasons.Remove(i);
            Save();
        }

        public void Save()
        {
            _dbContext.SaveChanges();
        }

        public void Update(Season item)
        {
            _dbContext.Entry(item).State = EntityState.Modified;
            Save();
        }
    }
}
