﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using UnivateProperties_API.Context;
using UnivateProperties_API.Model.Users;

namespace UnivateProperties_API.Repository.Users
{
    public class IndividualRepository : IRepository<Individual>
    {
        private readonly DataContext _dbContext;

        public IndividualRepository(DataContext dbContext)
        {
            _dbContext = dbContext;
        }

        public List<Individual> Get(Func<Individual, bool> where)
        {
            return _dbContext.Individuals.Where(where).ToList();
        }

        public List<Individual> GetAll()
        {
            return _dbContext.Individuals.ToList();
        }

        public Individual GetDetailed(Func<Individual, bool> first)
        {
            var item = _dbContext.Individuals.FirstOrDefault(first);
            IndividualRepository account = new IndividualRepository(_dbContext);
            item = GetDetailedObject(item, account);
            return item;
        }

        private Individual GetDetailedObject(Individual item, IndividualRepository repo)
        {
            item = repo.GetDetailed(x => x.Id == item.Id);
            return item;
        }

        public void Insert(Individual item)
        {
            _dbContext.Add(item);
            Save();
        }

        public void Insert(IEnumerable<Individual> item)
        {
            _dbContext.Add(item);
            Save();
        }

        public void Remove(Individual item)
        {
            var i = _dbContext.Individuals.Find(item);
            _dbContext.Individuals.Remove(i);
            Save();
        }

        public void Remove(IEnumerable<Individual> items)
        {
            foreach (var item in items)
            {
                Individual i = _dbContext.Individuals.Find(item);
                _dbContext.Individuals.Remove(i);
            }
            Save();
        }

        public void RemoveAtId(int item)
        {
            var i = _dbContext.Individuals.Find(item);
            _dbContext.Individuals.Remove(i);
            Save();
        }

        //Updates in DB
        public void Update(Individual item)
        {
            _dbContext.Entry(item).State = EntityState.Modified;
            Save();
        }

        public void Save()
        {
            _dbContext.SaveChanges();
        }

        public List<Individual> GetDetailedAll()
        {
            // TODO: GetDetailed
            throw new NotImplementedException();
        }
    }
}
